@@ -50,7 +50,7 @@ class Function(ABC):
         if hasattr(self, "_memsave") and getattr(self, "memsave", False):
             self._memsave(parents_for_hooks, output)
         # Optional per-op memory and timing logging (enabled only inside MemoryMonitor)
-        maybe_log_op_memory(op_name, parents_for_hooks, output_data, timing_context)
+        maybe_log_op_memory(f"{op_name}_fwd", parents_for_hooks, output_data, timing_context)
         return output
 
     @abstractmethod
