@@ -15,9 +15,77 @@ try:
     from nvidia.dali import pipeline_def, Pipeline
     import nvidia.dali.fn as fn
     import nvidia.dali.types as types
-    from nvidia.dali.plugin.pytorch import DALIGenericIterator, LastBatchPolicy
+    # FIX 1: Import LastBatchPolicy from its location in older DALI versions
+    from nvidia.dali.plugin.base_iterator import LastBatchPolicy
     from nvidia.dali.backend import PreallocateDeviceMemory, PreallocatePinnedMemory
     DALI_AVAILABLE = True
+
+    class DALIGenericIterator:
+        """
+        A generic DALI iterator that is not tied to any specific framework.
+        It iterates over a DALI pipeline and yields batches of data.
+        """
+        def __init__(self,
+                     pipeline: Pipeline,
+                     output_map: List[str],
+                     last_batch_policy: LastBatchPolicy,
+                     auto_reset: bool = False,
+                     reader_name: Optional[str] = None,
+                     prepare_first_batch: bool = True):
+            self._pipeline = pipeline
+            self._output_map = output_map
+            self._last_batch_policy = last_batch_policy
+            self._auto_reset = auto_reset
+            self._reader_name = reader_name
+
+            # Try to find reader name if not provided
+            if not self._reader_name:
+                readers = [op.name for op in self._pipeline.ops if "readers" in op.spec.name]
+                if len(readers) == 1:
+                    self._reader_name = readers[0]
+                else:
+                    raise ValueError(f"Could not automatically determine the reader name. "
+                                     f"Found {len(readers)} readers: {readers}. Please specify 'reader_name'.")
+
+            self._size = self._pipeline.epoch_size(self._reader_name)
+            self._batch_size = self._pipeline.max_batch_size
+            
+            if self._last_batch_policy == LastBatchPolicy.DROP:
+                self._num_batches = self._size // self._batch_size
+            else:  # PARTIAL or FILL
+                self._num_batches = math.ceil(self._size / self._batch_size)
+            
+            self._counter = 0
+
+        def __iter__(self):
+            return self
+
+        def __len__(self):
+            return self._num_batches
+
+        def __next__(self):
+            if self._counter >= self._num_batches:
+                if self._auto_reset:
+                    self.reset()
+                raise StopIteration
+
+            try:
+                outputs = self._pipeline.run()
+                self._counter += 1
+                
+                # FIX 2: Correctly map the tuple of outputs to the output_map keys.
+                batch_dict = {key: outputs[i] for i, key in enumerate(self._output_map)}
+                return [batch_dict]
+
+            except StopIteration:
+                if self._auto_reset:
+                    self.reset()
+                raise
+
+        def reset(self):
+            self._pipeline.reset()
+            self._counter = 0
+
 except ImportError:
     DALI_AVAILABLE = False
     print("INFO: NVIDIA DALI not available. Falling back to OpenCV-based implementation.")
@@ -183,121 +251,38 @@ class ImageFolder(Dataset):
         if not DALI_AVAILABLE:
             return None
             
-        # If user provided an entire DALI Pipeline instance, return it directly.
         if isinstance(self.img_transform, Pipeline):
             return self.img_transform
 
-        # Pre-compute constants
         is_gpu = device == "gpu"
-        device_id = 0 if is_gpu else None
-        prefetch_depth = 3 if is_gpu else 2
         h, w = self.img_shape or (224, 224)
-        is_grayscale = self.img_mode.upper() in ("L", "GRAY", "GREY", "GRAYSCALE")
         
-        # Pre-compute normalization parameters if needed
-        imagenet_mean = [0.485 * 255, 0.456 * 255, 0.406 * 255] if self.img_normalize else None
-        imagenet_std = [0.229 * 255, 0.224 * 255, 0.225 * 255] if self.img_normalize else None
-        
-        @pipeline_def(
-            batch_size=batch_size, 
-            num_threads=num_threads, 
-            device_id=device_id, 
-            seed=seed, 
-            prefetch_queue_depth=prefetch_depth,
-            enable_memory_stats=False  # Disable for better performance
-        )
+        @pipeline_def(batch_size=batch_size, num_threads=num_threads, device_id=0 if is_gpu else None, seed=seed)
         def image_pipeline():
-            # Create file reader with optimized buffer sizes
-            initial_fill = min(batch_size * 4, len(self.images))  # Don't exceed dataset size
-            images, labels = fn.readers.file(
-                files=self.images,
-                labels=self.numeric_targets,
-                random_shuffle=shuffle,
-                initial_fill=initial_fill,
-                read_ahead=True,  # Enable read-ahead for better I/O
-                name="Reader"
-            )
-
-            # Optimized decoding strategy
-            if is_gpu:
-                # Decode on GPU with mixed device
-                images = fn.decoders.image(
-                    images, 
-                    device="mixed",
-                    output_type=types.RGB
-                )
-            else:
-                # Decode and resize on CPU
-                images = fn.decoders.image(
-                    images,
-                    device="cpu", 
-                    output_type=types.RGB
-                )
-            images = fn.resize(
-                images,
-                resize_x=w,
-                resize_y=h,
-                interp_type=types.INTERP_LINEAR
-            )
-
-            # Handle grayscale conversion early if needed
-            if is_grayscale:
-                images = fn.color_space_conversion(
-                    images, 
-                    image_type=types.RGB, 
-                    output_type=types.GRAY
-                )
-
-            # Apply custom transforms if provided (with error handling)
+            images, labels = fn.readers.file(files=self.images, labels=self.numeric_targets, random_shuffle=shuffle, name="Reader")
+            images = fn.decoders.image(images, device="mixed" if is_gpu else "cpu", output_type=types.RGB)
+            images = fn.resize(images, resize_x=w, resize_y=h, interp_type=types.INTERP_LINEAR)
+            
             if self.img_transform and callable(self.img_transform):
-                try:
-                    images = self.img_transform(images)
-                except Exception as e:
-                    # Log warning but continue - could use logging instead of print
-                    print(f"WARNING: DALI transform failed: {e}. Skipping transform.")
-
-            # Optimized normalization and formatting
+                images = self.img_transform(images)
+            
             if self.img_normalize:
-                # Single operation for normalize + layout conversion
                 images = fn.crop_mirror_normalize(
-                    images,
-                    dtype=types.FLOAT,
-                    output_layout="CHW" if self.chw else "HWC",
-                    mean=imagenet_mean,
-                    std=imagenet_std,
-                    crop=(h, w),  # Explicit crop size
-                    mirror=0  # Disable mirroring for inference
+                    images, dtype=types.FLOAT, output_layout="CHW" if self.chw else "HWC",
+                    mean=[0.485 * 255, 0.456 * 255, 0.406 * 255],
+                    std=[0.229 * 255, 0.224 * 255, 0.225 * 255]
                 )
             else:
-                # Efficient cast and transpose
-                images = fn.cast(images, dtype=types.FLOAT)
-                if self.chw:
-                    images = fn.transpose(images, perm=[2, 0, 1])
+                if self.chw: images = fn.transpose(images, perm=[2, 0, 1])
             
-            # Efficient one-hot encoding
             if self.one_hot_targets:
-                # Use int32 for better performance than default int64
-                labels = fn.cast(labels, dtype=types.INT32)
                 labels = fn.one_hot(labels, num_classes=self.num_classes)
-            
-            # Move to GPU at the end to minimize transfers
-            if is_gpu:
-                images = images.gpu()
-                labels = labels.gpu()
-            
+
+            # A DALI pipeline function should return the DataNode objects directly.
             return images, labels
 
-        # Build and return the pipeline
         pipeline = image_pipeline()
-        
-        # Optional: warm up the pipeline for consistent timing
-        # This can be enabled based on a parameter
-        try:
-            pipeline.build()
-        except Exception as e:
-            print(f"WARNING: Pipeline build failed: {e}")
-            return None
-            
+        pipeline.build()
         return pipeline
 
     def _apply_img_transform(self, arr: np.ndarray) -> np.ndarray:
@@ -310,7 +295,6 @@ class ImageFolder(Dataset):
                   "Cannot apply Pipeline to ndarray; skipping transform.")
             return arr
             
-        # Try Albumentations-style call
         try:
             out = self.img_transform(image=arr)
             if isinstance(out, dict) and "image" in out:
@@ -319,7 +303,6 @@ class ImageFolder(Dataset):
         except TypeError:
             pass
 
-        # Fallback: plain callable expecting ndarray
         try:
             return self.img_transform(arr)
         except Exception as e:
@@ -328,7 +311,6 @@ class ImageFolder(Dataset):
 
     def _load_image_opencv(self, path: str) -> np.ndarray:
         """Load image using OpenCV (fallback implementation)"""
-        # OpenCV-only fast decode/resize
         mode = (self.img_mode or "RGB").upper()
         if mode in ("L", "GRAY", "GREY", "GRAYSCALE"):
             flag = cv2.IMREAD_GRAYSCALE
@@ -346,13 +328,11 @@ class ImageFolder(Dataset):
         if arr is None:
             raise ValueError(f"Failed to read image: {path}")
 
-        # Convert channel order to match RGB/RGBA expectations
         if mode == "RGB" and arr.ndim == 3 and arr.shape[2] == 3:
             arr = cv2.cvtColor(arr, cv2.COLOR_BGR2RGB)
         elif mode == "RGBA" and arr.ndim == 3 and arr.shape[2] == 4:
             arr = cv2.cvtColor(arr, cv2.COLOR_BGRA2RGBA)
 
-        # Resize if requested
         if self.img_shape is not None:
             h, w = self.img_shape
             arr = cv2.resize(arr, (int(w), int(h)), interpolation=cv2.INTER_LINEAR)
@@ -360,15 +340,12 @@ class ImageFolder(Dataset):
         if arr.ndim == 2:
             arr = arr[:, :, None]
         
-        # Apply transforms
         if self.img_transform:
             arr = self._apply_img_transform(arr)
         
-        # Convert to CHW if requested
         if self.chw and arr.ndim == 3:
             arr = np.transpose(arr, (2, 0, 1))
         
-        # Normalize
         if self.img_normalize:
             arr = arr.astype(np.float32) / 255.0
         else:
@@ -380,15 +357,11 @@ class ImageFolder(Dataset):
         """Get single item - uses OpenCV fallback"""
         img_path = self.images[idx]
         target = self.numeric_targets[idx]
-        
-        # Load image using OpenCV
         image = self._load_image_opencv(img_path)
         
-        # Apply one-hot encoding if requested
         if self.one_hot_targets:
-            # Use our precomputed one-hot mapping
             target = self.one_hot_mapping[target]
-            target_dtype = float32  # One-hot targets should be float
+            target_dtype = float32
         else:
             target_dtype = self.target_dtype
         
@@ -399,10 +372,7 @@ class ImageFolder(Dataset):
         rng = random.Random(seed) if seed is not None else random.Random()
         combined = list(zip(self.images, self.targets, self.numeric_targets))
         rng.shuffle(combined)
-        self.images, self.targets, self.numeric_targets = zip(*combined)
-        self.images = list(self.images)
-        self.targets = list(self.targets)
-        self.numeric_targets = list(self.numeric_targets)
+        self.images, self.targets, self.numeric_targets = [list(t) for t in zip(*combined)]
 
     def __len__(self):
         return len(self.images)
@@ -416,25 +386,15 @@ class ImageFolder(Dataset):
                 f"classes={self.num_classes})")
 
     def __repr__(self):
-        shape = None
-        if len(self) > 0:
-            try:
-                image, _ = self[0]
-                shape = tuple(image.shape)
-            except Exception:
-                shape = None
-        
+        shape = tuple(self[0][0].shape) if len(self) > 0 else None
         return (f"ImageFolder(root='{self.root}', samples={len(self)}, "
-                f"classes={self.num_classes}, "
-                f"shape={shape}, img_dtype={self.img_dtype}, target_dtype={self.target_dtype}, "
-                f"mode='{self.img_mode}', normalize={self.img_normalize}, chw={self.chw}, "
-                f"one_hot_targets={self.one_hot_targets})")
+                f"classes={self.num_classes}, shape={shape})")
 
 
 class DataLoader:
     """
-    DataLoader that handles all data loading parameters including device, threading, and prefetching.
-    Automatically uses DALI when available and appropriate, with OpenCV fallback.
+    DataLoader that handles all data loading parameters, with an efficient DALI backend
+    and a multithreaded OpenCV fallback.
     """
     def __init__(
         self,
@@ -442,7 +402,7 @@ class DataLoader:
         batch_size: int = 32,
         shuffle: bool = True,
         device: str = "cpu",
-        num_workers: int = 4,
+        num_workers: int = None,
         prefetch_batches: int = 2,
         drop_last: bool = False,
         seed: Optional[int] = None,
@@ -451,7 +411,7 @@ class DataLoader:
         self.batch_size = batch_size
         self.shuffle = shuffle
         self.device = "gpu" if device in ["gpu", "cuda"] else "cpu"
-        self.num_workers = max(0, int(num_workers))
+        self.num_workers = os.cpu_count() if num_workers is None else max(0, int(num_workers))
         self.prefetch_batches = max(0, int(prefetch_batches))
         self.drop_last = drop_last
         self.seed = seed or 42
@@ -460,117 +420,80 @@ class DataLoader:
         self._dali_iter = None
         self._executor: Optional[ThreadPoolExecutor] = None
         
-        # Preallocate memory for better performance
         if DALI_AVAILABLE:
             try:
-                if self.device == "gpu":
-                    PreallocateDeviceMemory(int(0.5 * 1024**3), 0)  # 0.5GB GPU memory
-                PreallocatePinnedMemory(int(0.25 * 1024**3))  # 0.25GB pinned memory
-            except Exception as e:
-                print(f"WARNING: Memory preallocation failed: {e}")
-        
-        # Determine if we should use DALI
-        self.use_dali = (
-            DALI_AVAILABLE and 
-            isinstance(dataset, ImageFolder)
-        )
+                if self.device == "gpu": PreallocateDeviceMemory(int(0.5 * 1024**3), 0)
+                PreallocatePinnedMemory(int(0.25 * 1024**3))
+            except Exception as e: print(f"WARNING: DALI memory preallocation failed: {e}")
         
+        self.use_dali = DALI_AVAILABLE and isinstance(dataset, ImageFolder)
         if self.use_dali:
             self._init_dali_pipeline()
         
-        # Warn if GPU requested but not available
-        if device in ["gpu", "cuda"] and not self.use_dali:
+        if self.device == "gpu" and not self.use_dali:
             print("WARNING: GPU device requested but DALI not available. Using CPU fallback.")
             self.device = "cpu"
 
     def _init_dali_pipeline(self):
-        """Initialize the DALI pipeline"""
-        if not self.use_dali or not isinstance(self.dataset, ImageFolder):
-            return
-        
-        # Device-specific thread optimization
-        if self.device == "gpu":
-            num_threads = min(4, self.num_workers)  # Fewer threads for GPU
-        else:
-            num_threads = self.num_workers
-        
-        # Create pipeline
+        if not self.use_dali: return
         self._pipeline = self.dataset.get_dali_pipeline(
-            batch_size=self.batch_size,
-            shuffle=self.shuffle,
-            device=self.device,
-            num_threads=num_threads,
-            seed=self.seed
-        )
-        
-        if self._pipeline is not None:
-            # Build the pipeline
-            self._pipeline.build()
-            
-            # Set appropriate last batch policy (FIXED: use enum instead of string)
-            last_batch_policy = LastBatchPolicy.DROP if self.drop_last else LastBatchPolicy.PARTIAL
-            
-            # Create DALI iterator
-            self._dali_iter = DALIGenericIterator(
-                self._pipeline, 
-                output_map=["images", "labels"],
-                last_batch_policy=last_batch_policy,
-                auto_reset=False,
-                reader_name="Reader",
-                prepare_first_batch=True  # Preload first batch
-            )
-            print(f"DALI pipeline initialized: batch_size={self.batch_size}, device={self.device}, "
-                  f"num_threads={num_threads}")
+            batch_size=self.batch_size, shuffle=self.shuffle, device=self.device,
+            num_threads=self.num_workers, seed=self.seed)
+        if self._pipeline:
+            policy = LastBatchPolicy.DROP if self.drop_last else LastBatchPolicy.PARTIAL
+            self._dali_iter = DALIGenericIterator(self._pipeline, ["images", "labels"], policy, reader_name="Reader")
+            print(f"DALI pipeline initialized: batch_size={self.batch_size}, device={self.device}, num_threads={self.num_workers}")
 
     def __len__(self):
         n = len(self.dataset)
-        if self.drop_last:
-            return n // self.batch_size
-        return (n + self.batch_size - 1) // self.batch_size
+        return n // self.batch_size if self.drop_last else math.ceil(n / self.batch_size)
 
     def __iter__(self):
-        """Main iteration - uses DALI when available, falls back otherwise"""
-        if self.use_dali and self._dali_iter is not None:
+        if self.use_dali and self._dali_iter:
+            self._dali_iter.reset()
             return self._dali_iterator()
-        else:
-            return self._regular_iterator()
+        return self._regular_iterator()
+
+    def reset(self):
+        """
+        Resets the internal iterator, particularly for the DALI pipeline.
+        This allows re-iteration over the dataset from the beginning without
+        recreating the DataLoader.
+        """
+        if self.use_dali and self._dali_iter:
+            self._dali_iter.reset()
+        # For the regular iterator, a new one is created on each __iter__ call,
+        # so an explicit reset is not strictly necessary in the same way.
 
     def _dali_iterator(self):
-        """DALI-based iteration - maximum performance!"""
         for data in self._dali_iter:
-            # Extract data from DALI output
-            images = data[0]["images"]
-            labels = data[0]["labels"]
-            
-            # Convert to neurograd tensors
-            if self.device == "gpu":
-                # Use DLPack for zero-copy conversion
-                X = Tensor(xp.from_dlpack(images), dtype=self.dataset.img_dtype)
-                y = Tensor(xp.from_dlpack(labels), dtype=self.dataset.target_dtype)
-            else:
-                # Convert to numpy arrays
-                X = Tensor(images.numpy(), dtype=self.dataset.img_dtype)
-                y = Tensor(labels.numpy(), dtype=self.dataset.target_dtype)
+            # The pipeline outputs DALI Tensors which are extracted from the output dictionary.
+            images_tensor = data[0]["images"]
+            labels_tensor = data[0]["labels"]
             
+            # This simple, direct conversion is highly efficient.
+            if self.device == 'gpu':
+                X = Tensor(xp.from_dlpack(images_tensor), dtype=self.dataset.img_dtype)
+                y = Tensor(xp.from_dlpack(labels_tensor), dtype=self.dataset.target_dtype)
+            else: # CPU device
+                X = Tensor(np.array(images_tensor), dtype=self.dataset.img_dtype)
+                y = Tensor(np.array(labels_tensor), dtype=self.dataset.target_dtype)
+
             yield X, y
 
     def _regular_iterator(self):
-        """Regular iteration for non-DALI datasets with threading and prefetch"""
+        """Fallback iterator with threading and prefetching."""
         batches = list(self._batch_indices())
         window = deque()
         next_to_submit = 0
         total = len(batches)
 
-        # Prime the prefetch window
-        pre = self.prefetch_batches if self.prefetch_batches > 0 else 0
-        for _ in range(min(pre, total)):
+        for _ in range(min(self.prefetch_batches, total)):
             futs = self._schedule_batch(batches[next_to_submit])
             window.append(futs)
             next_to_submit += 1
 
-        # Iterate in order; keep the window full
-        for b in range(total):
-            # If window is empty (prefetch=0) or drained, schedule current batch now
+        for _ in range(total):
             if not window:
                 futs = self._schedule_batch(batches[next_to_submit])
                 window.append(futs)
@@ -578,95 +501,49 @@ class DataLoader:
 
             futs = window.popleft()
 
-            # Immediately schedule the next batch to keep the window full
-            if next_to_submit < total and len(window) < self.prefetch_batches:
+            if next_to_submit < total:
                 next_futs = self._schedule_batch(batches[next_to_submit])
                 window.append(next_futs)
                 next_to_submit += 1
 
-            # This blocks only if this batch isn't finished yet
-            X, y = self._gather_batch(futs)
-            yield X, y
+            yield self._gather_batch(futs)
 
     def _batch_indices(self):
-        """Generate batch indices with optional shuffling"""
         n = len(self.dataset)
         order = list(range(n))
         if self.shuffle:
-            rng = random.Random(self.seed) if self.seed is not None else random.Random()
-            rng.shuffle(order)
-        if self.drop_last:
-            limit = (n // self.batch_size) * self.batch_size
-        else:
-            limit = n
+            random.Random(self.seed).shuffle(order)
+        
+        limit = (n // self.batch_size) * self.batch_size if self.drop_last else n
         for start in range(0, limit, self.batch_size):
-            end = min(start + self.batch_size, limit)
+            end = min(start + self.batch_size, n)
             yield order[start:end]
 
     def _ensure_executor(self):
-        """Ensure thread executor is ready for fallback mode"""
         if self.num_workers > 0 and self._executor is None:
             self._executor = ThreadPoolExecutor(max_workers=self.num_workers)
 
     def _schedule_batch(self, idxs):
-        """Schedule all sample loads in this batch and return the list of futures"""
         if self.num_workers > 0:
             self._ensure_executor()
             return [self._executor.submit(self.dataset.__getitem__, i) for i in idxs]
-        else:
-            # synchronous path for num_workers=0
-            return [(self.dataset[i], None) for i in idxs]
+        return [self.dataset[i] for i in idxs]
 
     def _gather_batch(self, futures_or_results):
-        """Block until the batch is ready, then stack into (X, y) Tensors"""
         if self.num_workers > 0:
             batch = [f.result() for f in futures_or_results]
         else:
-            batch = [r for (r, _) in futures_or_results]
-
+            batch = futures_or_results
+        
         Xs, ys = zip(*batch)
-        X = Tensor(xp.stack([x.data for x in Xs], axis=0), dtype=Xs[0].dtype)
-        y = Tensor(xp.stack([y.data for y in ys], axis=0), dtype=ys[0].dtype)
-        return X, y
-
-    def __getitem__(self, idx):
-        """Get specific batch by index"""
-        if idx < 0 or idx >= len(self):
-            raise IndexError(f"Batch index {idx} out of range [0, {len(self)})")
-        
-        # Get all batch indices for consistent ordering
-        batches = list(self._batch_indices())
-        batch_idxs = batches[idx]
-        
-        # Load the batch synchronously
-        batch_data = [self.dataset[i] for i in batch_idxs]
-        Xs, ys = zip(*batch_data)
-        
-        # Stack into tensors
-        X = Tensor(xp.stack([x.data for x in Xs], axis=0), dtype=Xs[0].dtype)
-        y = Tensor(xp.stack([y.data for y in ys], axis=0), dtype=ys[0].dtype)
-        return X, y
-
-    def reset(self):
-        """Reset the iterator - useful for multi-epoch training"""
-        if self._pipeline is not None:
-            self._pipeline.reset()
+        return Tensor(xp.stack([x.data for x in Xs])), Tensor(xp.stack([y.data for y in ys]))
 
     def close(self):
-        """Clean up resources"""
         if self._executor is not None:
             self._executor.shutdown(wait=True)
             self._executor = None
 
-    def __repr__(self):
-        backend = "DALI" if self.use_dali else "OpenCV+Threading"
-        return (f"<DataLoader: {len(self)} batches, "
-                f"batch_size={self.batch_size}, shuffle={self.shuffle}, "
-                f"backend={backend}, device='{self.device}', "
-                f"num_workers={self.num_workers}, prefetch_batches={self.prefetch_batches}>")
 
-
-# DALI transform creation helper (only available when DALI is installed)
 def create_dali_transforms(
     device: str = "cpu",
     brightness: float = 0.0,
@@ -686,33 +563,23 @@ def create_dali_transforms(
         return None
         
     def apply_transforms(images):
-        # Color jitter
-        if brightness != 0.0 or contrast != 0.0 or saturation != 0.0 or hue != 0.0:
+        if brightness or contrast or saturation or hue:
             images = fn.color_twist(
-                images,
-                device=device,
-                brightness=fn.random.uniform(range=[-brightness, brightness]) if brightness > 0 else None,
-                contrast=fn.random.uniform(range=[1-contrast, 1+contrast]) if contrast > 0 else None,
-                saturation=fn.random.uniform(range=[1-saturation, 1+saturation]) if saturation > 0 else None,
-                hue=fn.random.uniform(range=[-hue, hue]) if hue > 0 else None,
+                images, device=device,
+                brightness=fn.random.uniform(range=[-brightness, brightness]),
+                contrast=fn.random.uniform(range=[1-contrast, 1+contrast]),
+                saturation=fn.random.uniform(range=[1-saturation, 1+saturation]),
+                hue=fn.random.uniform(range=[-hue, hue]),
             )
-        
-        # Flips
         if horizontal_flip_prob > 0.0:
             images = fn.flip(images, device=device, horizontal=fn.random.coin_flip(probability=horizontal_flip_prob))
         if vertical_flip_prob > 0.0:
             images = fn.flip(images, device=device, vertical=fn.random.coin_flip(probability=vertical_flip_prob))
-        
-        # Rotation
         if rotation_angle != 0.0:
             images = fn.rotate(
-                images,
-                device=device,
-                angle=fn.random.uniform(range=[-rotation_angle, rotation_angle]),
-                keep_size=True,
-                fill_value=0
+                images, device=device, angle=fn.random.uniform(range=[-rotation_angle, rotation_angle]),
+                keep_size=True, fill_value=0
             )
-        
         return images
     
     return apply_transforms
\ No newline at end of file
